
/*
 * Copyright (C) 2003-2005 Archie L. Cobbs and others.
 * All rights reserved.
 * 
 * This file is part of the JC virtual machine software.
 * 
 * JC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License,
 * or (at your option) any later version.
 * 
 * JC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with JC in the file "LGPL-2.1"; if not, write to:
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * $Id: i386_libjc.h,v 1.4 2005/02/23 22:48:16 archiecobbs Exp $
 */

#ifndef _ARCH_I386_LIBJC_H_
#define _ARCH_I386_LIBJC_H_

/*
 * Architecture-specific functions for i386 architecture.
 */

static inline int
_jc_compare_and_swap(volatile _jc_word *word, _jc_word old, _jc_word new)
{
	_jc_word previous;

	asm __volatile__ (
	    "lock\n"
	    "\tcmpxchgl %1,%2\n"
	    : "=a" (previous)
	    : "q" (new), "m" (*word), "0" (old)
	    : "memory");
	return previous == old;
}

static inline void
_jc_iflush(const void *mem, size_t len)
{
	/* nothing to do: i386 has coherent data and instruction caches */
}

static inline void
_jc_build_trampoline(u_char *code, _jc_method *method, const void *func)
{
	static const unsigned char template[_JC_TRAMPOLINE_SIZE] = {
	    0x8b, 0x44, 0x24, 0x04,		/* mov 0x4(%esp,1),%eax */
	    0xc7, 0x40, 0x33,			/* mov $0x44444444,0x33(%eax) */
		0x44, 0x44, 0x44, 0x44,
	    0xe9, 0x55, 0x55, 0x55, 0x55,	/* jmp pc+$0x55555555 */
	};

	/* Sanity checks */
	_JC_ASSERT(_JC_OFFSETOF(_jc_env, interp) < 0x80);
	_JC_ASSERT(sizeof(template) <= _JC_TRAMPOLINE_SIZE);

	/* Copy and modify template */
	memcpy(code, template, sizeof(template));
	code[6] = _JC_OFFSETOF(_jc_env, interp);	/* patch 0x33 */
	memcpy(code + 7, &method, sizeof(method));	/* patch 0x44444444 */
	func = (void *)((jint)func - (jint)(code + 16));
	memcpy(code + 12, &func, sizeof(func));		/* patch 0x55555555 */
}

static inline void
_jc_stack_frame_init(_jc_stack_frame *framep)
{
	*framep = NULL;
}

#define _jc_stack_frame_current(framep)					\
    do {								\
	*(framep) = __builtin_frame_address(0);				\
    } while (0)

static inline void
_jc_stack_frame_next(_jc_stack_frame *framep, const void **pcp)
{
	_jc_word *const ebp = *framep;

	*pcp = (const void *)ebp[1];	/* saved %eip is one slot above %ebp */
	*framep = (_jc_word *)ebp[0];	/* %ebp points to saved %ebp */
}

static inline jboolean
_jc_stack_frame_valid(_jc_stack_frame frame)
{
	return frame != NULL;
}

static inline jboolean
_jc_stack_frame_equal(_jc_stack_frame frame1, _jc_stack_frame frame2)
{
	return frame1 == frame2;
}

static inline const void *
_jc_stack_frame_sp(_jc_stack_frame frame)
{
	return (const void *)frame;
}

#ifdef __FreeBSD__

static inline const void *
_jc_mcontext_sp(const mcontext_t *mctx)
{
	return (const void *)mctx->mc_isp;
}

static inline const void *
_jc_mcontext_pc(const mcontext_t *mctx)
{
	return (const void *)mctx->mc_eip;
}

static inline _jc_stack_frame
_jc_mcontext_frame(const mcontext_t *mctx)
{
	return (_jc_word *)mctx->mc_ebp;
}

static inline const void *
_jc_signal_fault_address(int sig_num, siginfo_t *info, ucontext_t *uctx)
{
	return (const void *)uctx->uc_mcontext.mc_err;
}

#elif defined(__linux__)

static inline const void *
_jc_mcontext_sp(const mcontext_t *mctx)
{
	return (const void *)mctx->gregs[REG_ESP];
}

static inline const void *
_jc_mcontext_pc(const mcontext_t *mctx)
{
	return (const void *)mctx->gregs[REG_EIP];
}

static inline _jc_stack_frame
_jc_mcontext_frame(const mcontext_t *mctx)
{
	return (_jc_word *)mctx->gregs[REG_EBP];
}

static inline const void *
_jc_signal_fault_address(int sig_num, siginfo_t *info, ucontext_t *uctx)
{
	return (const void *)info->si_addr;
}

#else
#error "Unsupported O/S for i386 _jc_stack_frame_siginit()"
#endif

#endif	/* _ARCH_I386_LIBJC_H_ */

