
/*
 * Copyright (C) 2003-2005 Archie L. Cobbs and others.
 * All rights reserved.
 * 
 * This file is part of the JC virtual machine software.
 * 
 * JC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License,
 * or (at your option) any later version.
 * 
 * JC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with JC in the file "LGPL-2.1"; if not, write to:
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * $Id: exec.c,v 1.3 2004/07/05 21:03:27 archiecobbs Exp $
 */

#include "libjc.h"

/*
 * Execute a command.
 */
jint
_jc_exec_cmd(_jc_env *env, const char *fmt, ...)
{
	_jc_jvm *const vm = env->vm;
	char buf[512];
	va_list args;
	int result;
	char *cmd;
	FILE *fp;

	/* Print into buffer */
	va_start(args, fmt);
	cmd = _JC_FORMAT_STRINGV(env, fmt, args);
	va_end(args);
	if (cmd == NULL) {
		_jc_post_exception_info(env);
		return JNI_ERR;
	}

	/* Open command */
	if ((fp = popen(cmd, "r")) == NULL) {
		_jc_post_exception_msg(env, _JC_InternalError,
		    "cannot execute command \"%s\": popen: %s",
		    cmd, strerror(errno));
		return JNI_ERR;
	}

	/* Read results and print them out */
	while (fgets(buf, sizeof(buf), fp) != NULL)
		_jc_printf(vm, "%s", buf);

	/* Close the stream */
	result = pclose(fp);

	/* Check result */
	if (WIFEXITED(result)) {
		if (WEXITSTATUS(result) != 0) {
			_jc_post_exception_msg(env, _JC_InternalError,
			    "command \"%s\" exited with status %d",
			    cmd, WEXITSTATUS(result));
			return JNI_ERR;
		}
		return JNI_OK;
	} else if (WIFSIGNALED(result)) {
		_jc_post_exception_msg(env, _JC_InternalError,
		    "command \"%s\" was terminated by signal %d",
		    cmd, WTERMSIG(result));
		return JNI_ERR;
	}

	/* Should never get here */
	_JC_ASSERT(JNI_FALSE);
	return JNI_ERR;				/* silence compiler warning */
}

